import os
import csv
import glob
import sys
import json
from collections import defaultdict
import pandas as pd

from joblib import delayed, Parallel
from tqdm import tqdm
from collections import defaultdict


def write_list(data_list, path, ):
    with open(path, 'w') as f:
        writer = csv.writer(f, delimiter=',')
        for row in data_list:
            if row: writer.writerow(row)
    print('split saved to %s' % path)


def check_exists_homage(row, root, pra_to_prva, atomic_actions_by_pra_dict=None):

    pra, cat = row
    p, r, a = pra.split('_')
    prvas = [prva for prva in pra_to_prva[pra] if os.path.exists(os.path.join(root, p, prva))]

    result = []

    if atomic_actions_by_pra_dict is not None:
        # atomic
        for atomic_action in atomic_actions_by_pra_dict[pra]:
            result_row = []
            for prva in prvas:
                v = prva.split('_')[2]
                full_dirname = os.path.join(root, p, prva)
                # FIXME: add action length
                s, e, c, _ = atomic_action
                # FIXME: ignore actions whose end index is greater than video length. Check consistency!
                if e > len(os.listdir(full_dirname)):
                    continue
                c = '_'.join(c.split(' '))
                result_row += [full_dirname, s, e, c]
            result.append(result_row)
    else:
        # video-level, a single row
        for prva in prvas:
            v = prva.split('_')[2]
            full_dirname = os.path.join(root, p, prva)
            if os.path.exists(full_dirname):
                n_frames = len(glob.glob(os.path.join(full_dirname, '*.png')))
                # dir name, start frame, end frame, category
                result += [full_dirname, 0, n_frames - 1, cat]
        result = [result]

    return result


def get_split_homage(root, split_path, mode, atomic_actions_by_pra_dict=None):
    print('processing %s split ...' % mode)
    print('checking %s' % root)
    split_list = []
    split_content = pd.read_csv(split_path, header=None)
    pra_to_prva = defaultdict(list)
    for p in os.listdir(root):
        for prva in os.listdir(os.path.join(root, p)):
            p, r, v, a = prva.split('_')
            pra = '_'.join((p, r, a))
            pra_to_prva[pra].append(prva)

    split_list = Parallel(n_jobs=64)\
                 (delayed(check_exists_homage)(row, root, pra_to_prva, atomic_actions_by_pra_dict) \
                 for i, row in tqdm(split_content.iterrows(), total=len(split_content)))
    split_list = [j for i in split_list for j in i]
    return split_list


def _homage_get_atomic_actions(annotation_root):
    print('reading atomic action data...')
    atomic_actions_by_pra_dict = defaultdict(list)
    for p in tqdm(os.listdir(annotation_root)):
        for j in os.listdir(os.path.join(annotation_root, p)):
            r, a = j.split('_')[1:3]
            pra = '_'.join((p, r, a))
            with open(os.path.join(annotation_root, p, j)) as f:
                atomic_action_list = json.load(f)
            for d in atomic_action_list:
                atomic_actions_by_pra_dict[pra].append([d['frame_start'], d['frame_end'], d['class'], d['action_length']])
    return atomic_actions_by_pra_dict


def main_homage(f_root, in_root, out_root='./data/homage/', atomic=False):
    '''generate training/testing split, count number of available frames, save in csv'''

    train_split_path = os.path.join('list_with_activity_labels/train_list.csv')
    val_split_path = os.path.join('list_with_activity_labels/val_list.csv')
    #test_split_path = os.path.join('list_with_activity_labels/test_list.csv')

    atomic_actions_by_pra_dict = None

    if atomic:
        atomic_actions_by_pra_dict = _homage_get_atomic_actions(os.path.join('annotation_files/atomic_actions'))

    if not os.path.exists(out_root):
        os.makedirs(out_root)

    train_split = get_split_homage(f_root, train_split_path, 'train', atomic_actions_by_pra_dict)
    write_list(train_split, os.path.join(out_root, 'train_split{}.csv'.format('_atomic' if atomic else '')))

    val_split = get_split_homage(f_root, val_split_path, 'val', atomic_actions_by_pra_dict)
    write_list(val_split, os.path.join(out_root, 'val_split{}.csv'.format('_atomic' if atomic else '')))

    #test_split = get_split_homage(f_root, test_split_path, 'test', atomic_actions_by_pra_dict)
    #write_list(test_split, os.path.join(out_root, 'test_split{}.csv'.format('_atomic' if atomic else '')))


import argparse

if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('--homage', action='store_true')
    parser.add_argument('--homage-atomic', action='store_true')
    parser.add_argument('--dataset_path', default='./data', type=str)
    args = parser.parse_args()

    dataset_path = args.dataset_path

    if args.homage:
        main_homage(
            os.path.join('./div_frames'),
            os.path.join(dataset_path, 'homage'),
            out_root=os.path.join(dataset_path, 'homage'))

    if args.homage_atomic:
        main_homage(
            os.path.join('./div_frames'),
            os.path.join(dataset_path, 'homage'),
            out_root=os.path.join(dataset_path, 'homage'), atomic=True)
